@@ -1,4 +1,3 @@
-import cgi
 import io
 import os
 import tempfile
@@ -6,6 +5,7 @@ import typing
 from unittest import mock
 
 import pytest
+from multipart import MultipartParser
 
 import httpx
 from httpx._content import encode_request
@@ -26,15 +26,14 @@ def test_multipart(value, output):
     response = client.post("http://127.0.0.1:8000/", data=data, files=files)
     assert response.status_code == 200
 
-    # We're using the cgi module to verify the behavior here, which is a
-    # bit grungy, but sufficient just for our testing purposes.
     boundary = response.request.headers["Content-Type"].split("boundary=")[-1]
-    content_length = response.request.headers["Content-Length"]
-    pdict: dict = {
-        "boundary": boundary.encode("ascii"),
-        "CONTENT-LENGTH": content_length,
-    }
-    multipart = cgi.parse_multipart(io.BytesIO(response.content), pdict)
+    content_length = int(response.request.headers["Content-Length"])
+
+    multipart = {}
+    for part in MultipartParser(
+        io.BytesIO(response.content), boundary=boundary, content_length=content_length
+    ):
+        multipart[part.name] = [part.raw]
 
     # Note that the expected return type for text fields
     # appears to differs from 3.6 to 3.7+
@@ -42,6 +41,57 @@ def test_multipart(value, output):
     assert multipart["file"] == [b"<file content>"]
 
 
+@pytest.mark.parametrize(
+    "header",
+    [
+        "multipart/form-data; boundary=+++; charset=utf-8",
+        "multipart/form-data; charset=utf-8; boundary=+++",
+        "multipart/form-data; boundary=+++",
+        "multipart/form-data; boundary=+++ ;",
+        'multipart/form-data; boundary="+++"; charset=utf-8',
+        'multipart/form-data; charset=utf-8; boundary="+++"',
+        'multipart/form-data; boundary="+++"',
+        'multipart/form-data; boundary="+++" ;',
+    ],
+)
+def test_multipart_explicit_boundary(header: str) -> None:
+    client = httpx.Client(transport=httpx.MockTransport(echo_request_content))
+
+    files = {"file": io.BytesIO(b"<file content>")}
+    headers = {"content-type": header}
+    response = client.post("http://127.0.0.1:8000/", files=files, headers=headers)
+    assert response.status_code == 200
+
+    assert response.request.headers["Content-Type"] == header
+    content_length = int(response.request.headers["Content-Length"])
+
+    multipart = {}
+    for part in MultipartParser(
+        io.BytesIO(response.content), boundary="+++", content_length=content_length
+    ):
+        multipart[part.name] = [part.raw]
+
+    assert multipart["file"] == [b"<file content>"]
+
+
+@pytest.mark.parametrize(
+    "header",
+    [
+        "multipart/form-data; charset=utf-8",
+        "multipart/form-data; charset=utf-8; ",
+    ],
+)
+def test_multipart_header_without_boundary(header: str) -> None:
+    client = httpx.Client(transport=httpx.MockTransport(echo_request_content))
+
+    files = {"file": io.BytesIO(b"<file content>")}
+    headers = {"content-type": header}
+    response = client.post("http://127.0.0.1:8000/", files=files, headers=headers)
+
+    assert response.status_code == 200
+    assert response.request.headers["Content-Type"] == header
+
+
 @pytest.mark.parametrize(("key"), (b"abc", 1, 2.3, None))
 def test_multipart_invalid_key(key):
     client = httpx.Client(transport=httpx.MockTransport(echo_request_content))
@@ -79,15 +129,14 @@ def test_multipart_file_tuple():
     response = client.post("http://127.0.0.1:8000/", data=data, files=files)
     assert response.status_code == 200
 
-    # We're using the cgi module to verify the behavior here, which is a
-    # bit grungy, but sufficient just for our testing purposes.
     boundary = response.request.headers["Content-Type"].split("boundary=")[-1]
-    content_length = response.request.headers["Content-Length"]
-    pdict: dict = {
-        "boundary": boundary.encode("ascii"),
-        "CONTENT-LENGTH": content_length,
-    }
-    multipart = cgi.parse_multipart(io.BytesIO(response.content), pdict)
+    content_length = int(response.request.headers["Content-Length"])
+
+    multipart = {}
+    for part in MultipartParser(
+        io.BytesIO(response.content), boundary=boundary, content_length=content_length
+    ):
+        multipart[part.name] = [part.raw]
 
     # Note that the expected return type for text fields
     # appears to differs from 3.6 to 3.7+
@@ -287,18 +336,37 @@ def test_multipart_encode_files_allows_bytes_content() -> None:
         assert content == b"".join(stream)
 
 
-def test_multipart_encode_files_raises_exception_with_str_content() -> None:
-    files = {"file": ("test.txt", "<bytes content>", "text/plain")}
+def test_multipart_encode_files_allows_str_content() -> None:
+    files = {"file": ("test.txt", "<str content>", "text/plain")}
     with mock.patch("os.urandom", return_value=os.urandom(16)):
+        boundary = os.urandom(16).hex()
 
-        with pytest.raises(TypeError):
-            encode_request(data={}, files=files)  # type: ignore
+        headers, stream = encode_request(data={}, files=files)
+        assert isinstance(stream, typing.Iterable)
+
+        content = (
+            '--{0}\r\nContent-Disposition: form-data; name="file"; '
+            'filename="test.txt"\r\n'
+            "Content-Type: text/plain\r\n\r\n<str content>\r\n"
+            "--{0}--\r\n"
+            "".format(boundary).encode("ascii")
+        )
+        assert headers == {
+            "Content-Type": f"multipart/form-data; boundary={boundary}",
+            "Content-Length": str(len(content)),
+        }
+        assert content == b"".join(stream)
 
 
 def test_multipart_encode_files_raises_exception_with_StringIO_content() -> None:
     files = {"file": ("test.txt", io.StringIO("content"), "text/plain")}
-    with mock.patch("os.urandom", return_value=os.urandom(16)):
+    with pytest.raises(TypeError):
+        encode_request(data={}, files=files)  # type: ignore
 
+
+def test_multipart_encode_files_raises_exception_with_text_mode_file() -> None:
+    with tempfile.TemporaryFile(mode="w") as upload:
+        files = {"file": ("test.txt", upload, "text/plain")}
         with pytest.raises(TypeError):
             encode_request(data={}, files=files)  # type: ignore
 
